from __future__ import absolute_import
from __future__ import print_function
import os
import sys
from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer

codedir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/verilogcode/'

expected = """\
(Bind dest:TOP.cnt \
tree:(Branch Cond:(Terminal TOP.RST) \
True:(IntConst 'd0) \
False:(Operator Plus Next:(Terminal TOP.cnt),(IntConst 'd1))))
"""

def test():
    filelist = [codedir + 'decimal.v']
    topmodule = 'TOP'
    noreorder = False
    nobind = False
    include = None
    define = None
    
    analyzer = VerilogDataflowAnalyzer(filelist, topmodule,
                                       noreorder=noreorder,
                                       nobind=nobind,
                                       preprocess_include=include,
                                       preprocess_define=define)
    analyzer.generate()

    directives = analyzer.get_directives()
    instances = analyzer.getInstances()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    output = []
    output.append(list(binddict.values())[0][0].tostr())
    output.append('\n')
            
    rslt = ''.join(output)

    print(rslt)
    assert(expected == rslt)

if __name__ == '__main__':
    test()



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer

codedir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/verilogcode/'

expected = """\
(Bind dest:TOP.WD tree:(IntConst 4))
(Bind dest:TOP._rn0_j tree:(IntConst 0))
(Bind dest:TOP._rn1_j tree:(Operator Plus Next:'d0,(IntConst 1)))
(Bind dest:TOP._rn2_j tree:(Operator Plus Next:'d1,(IntConst 1)))
(Bind dest:TOP._rn3_j tree:(Operator Plus Next:'d2,(IntConst 1)))
(Bind dest:TOP._rn4_j tree:(Operator Plus Next:'d3,(IntConst 1)))
(Bind dest:TOP.j tree:(Terminal TOP._rn4_j))
(Bind dest:TOP.md_generate0.ge_for0[0].subloop.ge_if0._subt.tmp \
tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) \
True:(Operator Unot Next:(Partselect Var:(Terminal TOP.subin) MSB:'d0 LSB:'d0))))
(Bind dest:TOP.subout msb:'d0 lsb:'d0 \
tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) \
True:(Partselect Var:(Terminal TOP.subin) MSB:'d0 LSB:'d0)))
(Bind dest:TOP.subout msb:'d1 lsb:'d1 \
tree:(Branch Cond:(Operator Eq Next:'d1,(IntConst 0)) \
False:(Branch Cond:(Operator Eq Next:'d1,(IntConst 1)) \
True:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[0].subloop.ge_if0._subt.tmp),(Partselect Var:(Terminal TOP.subin) MSB:'d1 LSB:'d1)))))
(Bind dest:TOP.subout msb:'d2 lsb:'d2 \
tree:(Branch Cond:(Operator Eq Next:'d2,(IntConst 0)) \
False:(Branch Cond:(Operator Eq Next:'d2,(IntConst 1)) \
False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[1].subloop.ge_if1_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.subin) MSB:'d2 LSB:'d2)))))
(Bind dest:TOP.subout msb:'d3 lsb:'d3 \
tree:(Branch Cond:(Operator Eq Next:'d3,(IntConst 0)) \
False:(Branch Cond:(Operator Eq Next:'d3,(IntConst 1)) \
False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[2].subloop.ge_if3_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.subin) MSB:'d3 LSB:'d3)))))
"""

def test():
    filelist = [codedir + 'generate.v']
    topmodule = 'TOP'
    noreorder = False
    nobind = False
    include = None
    define = None
    
    analyzer = VerilogDataflowAnalyzer(filelist, topmodule,
                                       noreorder=noreorder,
                                       nobind=nobind,
                                       preprocess_include=include,
                                       preprocess_define=define)
    analyzer.generate()

    directives = analyzer.get_directives()
    instances = analyzer.getInstances()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    output = []
            
    for bk, bv in sorted(binddict.items(), key=lambda x:str(x[0])):
        for bvi in bv:
            output.append(bvi.tostr())
            output.append('\n')
            
    rslt = ''.join(output)

    print(rslt)
    assert(expected == rslt)

if __name__ == '__main__':
    test()



from __future__ import absolute_import
from __future__ import print_function
import os
import sys
from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer

codedir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/verilogcode/'

expected = """\
(Bind dest:TOP.LEN tree:(IntConst 8))
(Bind dest:TOP.OUT0 tree:(Pointer Var:(Terminal TOP.out) PTR:(IntConst 0)))
(Bind dest:TOP.OUT1 tree:(Pointer Var:(Terminal TOP.out) PTR:(IntConst 1)))
(Bind dest:TOP.OUT2 tree:(Pointer Var:(Terminal TOP.out) PTR:(IntConst 2)))
(Bind dest:TOP.OUT3 tree:(Pointer Var:(Terminal TOP.out) PTR:(IntConst 3)))
(Bind dest:TOP.OUT4 tree:(Pointer Var:(Terminal TOP.out) PTR:(IntConst 4)))
(Bind dest:TOP.WA tree:(IntConst 32))
(Bind dest:TOP.WD tree:(IntConst 4))
(Bind dest:TOP._rn0_i tree:(IntConst 0))
(Bind dest:TOP._rn12_i tree:(Operator Plus Next:'d1,(IntConst 1)))
(Bind dest:TOP._rn18_i tree:(Operator Plus Next:'d2,(IntConst 1)))
(Bind dest:TOP._rn24_i tree:(Operator Plus Next:'d3,(IntConst 1)))
(Bind dest:TOP._rn30_i tree:(Operator Plus Next:'d4,(IntConst 1)))
(Bind dest:TOP._rn6_i tree:(Operator Plus Next:'d0,(IntConst 1)))
(Bind dest:TOP.cnt tree:(Branch Cond:(Operator Ulnot Next:(Terminal TOP.RST_X)) True:(IntConst 0) False:(Branch Cond:(Operator LessThan Next:(Terminal TOP.cnt),(IntConst 4)) True:(Operator Plus Next:(Terminal TOP.cnt),(IntConst 1)) False:(IntConst 0))))
(Bind dest:TOP.i tree:(Terminal TOP._rn30_i))
(Bind dest:TOP.in ptr:'d0 tree:(Terminal TOP.IN0))
(Bind dest:TOP.in ptr:'d1 tree:(Terminal TOP.IN1))
(Bind dest:TOP.in ptr:'d2 tree:(Terminal TOP.IN2))
(Bind dest:TOP.in ptr:'d3 tree:(Terminal TOP.IN3))
(Bind dest:TOP.in ptr:'d4 tree:(Terminal TOP.IN4))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub.CLK tree:(Terminal TOP.CLK))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub.RST_X tree:(Terminal TOP.RST_X))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub.WD tree:(Terminal TOP.WD))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub._rn1_j tree:(IntConst 0))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub._rn2_j tree:(Operator Plus Next:'d0,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub._rn3_j tree:(Operator Plus Next:'d1,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub._rn4_j tree:(Operator Plus Next:'d2,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub._rn5_j tree:(Operator Plus Next:'d3,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub.j tree:(Terminal TOP.md_generate0.ge_for0[0].loop.sub._rn5_j))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub.md_generate1.ge_for1[0].subloop.ge_if0._subt.tmp tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) True:(Operator Unot Next:(Partselect Var:(Terminal TOP.md_generate0.ge_for0[0].loop.sub.subin) MSB:'d0 LSB:'d0))))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub.subin tree:(Pointer Var:(Terminal TOP.in) PTR:(Terminal TOP.cnt)))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub.subout msb:'d0 lsb:'d0 tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) True:(Partselect Var:(Terminal TOP.md_generate0.ge_for0[0].loop.sub.subin) MSB:'d0 LSB:'d0)))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub.subout msb:'d1 lsb:'d1 tree:(Branch Cond:(Operator Eq Next:'d1,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d1,(IntConst 1)) True:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[0].loop.sub.md_generate1.ge_for1[0].subloop.ge_if0._subt.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[0].loop.sub.subin) MSB:'d1 LSB:'d1)))))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub.subout msb:'d2 lsb:'d2 tree:(Branch Cond:(Operator Eq Next:'d2,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d2,(IntConst 1)) False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[0].loop.sub.md_generate1.ge_for1[1].subloop.ge_if1_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[0].loop.sub.subin) MSB:'d2 LSB:'d2)))))
(Bind dest:TOP.md_generate0.ge_for0[0].loop.sub.subout msb:'d3 lsb:'d3 tree:(Branch Cond:(Operator Eq Next:'d3,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d3,(IntConst 1)) False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[0].loop.sub.md_generate1.ge_for1[2].subloop.ge_if3_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[0].loop.sub.subin) MSB:'d3 LSB:'d3)))))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub.CLK tree:(Terminal TOP.CLK))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub.RST_X tree:(Terminal TOP.RST_X))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub.WD tree:(Terminal TOP.WD))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub._rn10_j tree:(Operator Plus Next:'d2,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub._rn11_j tree:(Operator Plus Next:'d3,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub._rn7_j tree:(IntConst 0))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub._rn8_j tree:(Operator Plus Next:'d0,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub._rn9_j tree:(Operator Plus Next:'d1,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub.j tree:(Terminal TOP.md_generate0.ge_for0[1].loop.sub._rn11_j))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub.md_generate2.ge_for2[0].subloop.ge_if7._subt.tmp tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) True:(Operator Unot Next:(Partselect Var:(Terminal TOP.md_generate0.ge_for0[1].loop.sub.subin) MSB:'d0 LSB:'d0))))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub.subin tree:(Pointer Var:(Terminal TOP.in) PTR:(Terminal TOP.cnt)))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub.subout msb:'d0 lsb:'d0 tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) True:(Partselect Var:(Terminal TOP.md_generate0.ge_for0[1].loop.sub.subin) MSB:'d0 LSB:'d0)))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub.subout msb:'d1 lsb:'d1 tree:(Branch Cond:(Operator Eq Next:'d1,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d1,(IntConst 1)) True:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[1].loop.sub.md_generate2.ge_for2[0].subloop.ge_if7._subt.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[1].loop.sub.subin) MSB:'d1 LSB:'d1)))))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub.subout msb:'d2 lsb:'d2 tree:(Branch Cond:(Operator Eq Next:'d2,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d2,(IntConst 1)) False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[1].loop.sub.md_generate2.ge_for2[1].subloop.ge_if8_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[1].loop.sub.subin) MSB:'d2 LSB:'d2)))))
(Bind dest:TOP.md_generate0.ge_for0[1].loop.sub.subout msb:'d3 lsb:'d3 tree:(Branch Cond:(Operator Eq Next:'d3,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d3,(IntConst 1)) False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[1].loop.sub.md_generate2.ge_for2[2].subloop.ge_if10_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[1].loop.sub.subin) MSB:'d3 LSB:'d3)))))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub.CLK tree:(Terminal TOP.CLK))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub.RST_X tree:(Terminal TOP.RST_X))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub.WD tree:(Terminal TOP.WD))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub._rn13_j tree:(IntConst 0))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub._rn14_j tree:(Operator Plus Next:'d0,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub._rn15_j tree:(Operator Plus Next:'d1,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub._rn16_j tree:(Operator Plus Next:'d2,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub._rn17_j tree:(Operator Plus Next:'d3,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub.j tree:(Terminal TOP.md_generate0.ge_for0[2].loop.sub._rn17_j))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub.md_generate3.ge_for3[0].subloop.ge_if14._subt.tmp tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) True:(Operator Unot Next:(Partselect Var:(Terminal TOP.md_generate0.ge_for0[2].loop.sub.subin) MSB:'d0 LSB:'d0))))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub.subin tree:(Pointer Var:(Terminal TOP.in) PTR:(Terminal TOP.cnt)))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub.subout msb:'d0 lsb:'d0 tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) True:(Partselect Var:(Terminal TOP.md_generate0.ge_for0[2].loop.sub.subin) MSB:'d0 LSB:'d0)))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub.subout msb:'d1 lsb:'d1 tree:(Branch Cond:(Operator Eq Next:'d1,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d1,(IntConst 1)) True:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[2].loop.sub.md_generate3.ge_for3[0].subloop.ge_if14._subt.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[2].loop.sub.subin) MSB:'d1 LSB:'d1)))))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub.subout msb:'d2 lsb:'d2 tree:(Branch Cond:(Operator Eq Next:'d2,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d2,(IntConst 1)) False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[2].loop.sub.md_generate3.ge_for3[1].subloop.ge_if15_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[2].loop.sub.subin) MSB:'d2 LSB:'d2)))))
(Bind dest:TOP.md_generate0.ge_for0[2].loop.sub.subout msb:'d3 lsb:'d3 tree:(Branch Cond:(Operator Eq Next:'d3,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d3,(IntConst 1)) False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[2].loop.sub.md_generate3.ge_for3[2].subloop.ge_if17_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[2].loop.sub.subin) MSB:'d3 LSB:'d3)))))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub.CLK tree:(Terminal TOP.CLK))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub.RST_X tree:(Terminal TOP.RST_X))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub.WD tree:(Terminal TOP.WD))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub._rn19_j tree:(IntConst 0))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub._rn20_j tree:(Operator Plus Next:'d0,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub._rn21_j tree:(Operator Plus Next:'d1,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub._rn22_j tree:(Operator Plus Next:'d2,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub._rn23_j tree:(Operator Plus Next:'d3,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub.j tree:(Terminal TOP.md_generate0.ge_for0[3].loop.sub._rn23_j))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub.md_generate4.ge_for4[0].subloop.ge_if21._subt.tmp tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) True:(Operator Unot Next:(Partselect Var:(Terminal TOP.md_generate0.ge_for0[3].loop.sub.subin) MSB:'d0 LSB:'d0))))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub.subin tree:(Pointer Var:(Terminal TOP.in) PTR:(Terminal TOP.cnt)))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub.subout msb:'d0 lsb:'d0 tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) True:(Partselect Var:(Terminal TOP.md_generate0.ge_for0[3].loop.sub.subin) MSB:'d0 LSB:'d0)))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub.subout msb:'d1 lsb:'d1 tree:(Branch Cond:(Operator Eq Next:'d1,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d1,(IntConst 1)) True:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[3].loop.sub.md_generate4.ge_for4[0].subloop.ge_if21._subt.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[3].loop.sub.subin) MSB:'d1 LSB:'d1)))))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub.subout msb:'d2 lsb:'d2 tree:(Branch Cond:(Operator Eq Next:'d2,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d2,(IntConst 1)) False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[3].loop.sub.md_generate4.ge_for4[1].subloop.ge_if22_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[3].loop.sub.subin) MSB:'d2 LSB:'d2)))))
(Bind dest:TOP.md_generate0.ge_for0[3].loop.sub.subout msb:'d3 lsb:'d3 tree:(Branch Cond:(Operator Eq Next:'d3,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d3,(IntConst 1)) False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[3].loop.sub.md_generate4.ge_for4[2].subloop.ge_if24_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[3].loop.sub.subin) MSB:'d3 LSB:'d3)))))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub.CLK tree:(Terminal TOP.CLK))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub.RST_X tree:(Terminal TOP.RST_X))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub.WD tree:(Terminal TOP.WD))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub._rn25_j tree:(IntConst 0))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub._rn26_j tree:(Operator Plus Next:'d0,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub._rn27_j tree:(Operator Plus Next:'d1,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub._rn28_j tree:(Operator Plus Next:'d2,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub._rn29_j tree:(Operator Plus Next:'d3,(IntConst 1)))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub.j tree:(Terminal TOP.md_generate0.ge_for0[4].loop.sub._rn29_j))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub.md_generate5.ge_for5[0].subloop.ge_if28._subt.tmp tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) True:(Operator Unot Next:(Partselect Var:(Terminal TOP.md_generate0.ge_for0[4].loop.sub.subin) MSB:'d0 LSB:'d0))))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub.subin tree:(Pointer Var:(Terminal TOP.in) PTR:(Terminal TOP.cnt)))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub.subout msb:'d0 lsb:'d0 tree:(Branch Cond:(Operator Eq Next:'d0,(IntConst 0)) True:(Partselect Var:(Terminal TOP.md_generate0.ge_for0[4].loop.sub.subin) MSB:'d0 LSB:'d0)))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub.subout msb:'d1 lsb:'d1 tree:(Branch Cond:(Operator Eq Next:'d1,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d1,(IntConst 1)) True:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[4].loop.sub.md_generate5.ge_for5[0].subloop.ge_if28._subt.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[4].loop.sub.subin) MSB:'d1 LSB:'d1)))))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub.subout msb:'d2 lsb:'d2 tree:(Branch Cond:(Operator Eq Next:'d2,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d2,(IntConst 1)) False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[4].loop.sub.md_generate5.ge_for5[1].subloop.ge_if29_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[4].loop.sub.subin) MSB:'d2 LSB:'d2)))))
(Bind dest:TOP.md_generate0.ge_for0[4].loop.sub.subout msb:'d3 lsb:'d3 tree:(Branch Cond:(Operator Eq Next:'d3,(IntConst 0)) False:(Branch Cond:(Operator Eq Next:'d3,(IntConst 1)) False:(Operator Xor Next:(Terminal TOP.md_generate0.ge_for0[4].loop.sub.md_generate5.ge_for5[2].subloop.ge_if31_ELSE._subf.tmp),(Partselect Var:(Terminal TOP.md_generate0.ge_for0[4].loop.sub.subin) MSB:'d3 LSB:'d3)))))
(Bind dest:TOP.out ptr:'d0 tree:(Terminal TOP.md_generate0.ge_for0[0].loop.sub.subout))
(Bind dest:TOP.out ptr:'d1 tree:(Terminal TOP.md_generate0.ge_for0[1].loop.sub.subout))
(Bind dest:TOP.out ptr:'d2 tree:(Terminal TOP.md_generate0.ge_for0[2].loop.sub.subout))
(Bind dest:TOP.out ptr:'d3 tree:(Terminal TOP.md_generate0.ge_for0[3].loop.sub.subout))
(Bind dest:TOP.out ptr:'d4 tree:(Terminal TOP.md_generate0.ge_for0[4].loop.sub.subout))
"""

def test():
    filelist = [codedir + 'generate_instance.v']
    topmodule = 'TOP'
    noreorder = False
    nobind = False
    include = None
    define = None
    
    analyzer = VerilogDataflowAnalyzer(filelist, topmodule,
                                       noreorder=noreorder,
                                       nobind=nobind,
                                       preprocess_include=include,
                                       preprocess_define=define)
    analyzer.generate()

    directives = analyzer.get_directives()
    instances = analyzer.getInstances()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    output = []
            
    for bk, bv in sorted(binddict.items(), key=lambda x:str(x[0])):
        for bvi in bv:
            output.append(bvi.tostr())
            output.append('\n')
            
    rslt = ''.join(output)

    print(rslt)
    assert(expected == rslt)

if __name__ == '__main__':
    test()



#-------------------------------------------------------------------------------
# codegen.py (Obsoleted)
#
# Code generator from dataflow
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------
from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import re

import pyverilog.utils.util as util
import pyverilog.utils.signaltype as signaltype
from pyverilog.dataflow.subset import VerilogSubset

class VerilogCodeGenerator(VerilogSubset):
    def __init__(self, topmodule, terms, binddict, 
                 resolved_terms, resolved_binddict, constlist,
                 modulename='Subset', enable_name='HT_enable', num_indent=2, flat=True):
        VerilogSubset.__init__(self, topmodule, terms, binddict, 
                               resolved_terms, resolved_binddict, constlist)
        self.modulename = modulename
        self.enable_name = enable_name
        self.num_indent = num_indent
        self.flat = flat

    ############################################################################
    def addBind(self, name, bind):
        if name in self.binddict:
            self.binddict[name] += (bind,)
        else:
            self.binddict[name] = (bind,)
        if name in self.resolved_binddict:
            self.resolved_binddict[name] += (bind,)
        else:
            self.resolved_binddict[name] = (bind,)

    def addTerm(self, name, term):
        self.terms[name] = term
        self.resolved_terms[name] = term

    ############################################################################
    def _input(self, name):
        return 'input ' + name + ';\n'
    def _output(self, name):
        return 'output ' + name + ';\n'
    def _inout(self, name):
        return 'inout ' + name + ';\n'
    def _wire(self, name):
        return 'wire ' + name + ';\n'

    ############################################################################
    def _modulehead(self, terms):
        modulehead = ''
        modulehead += '`default_nettype none\n'
        modulehead += 'module ' + self.modulename + '('
        modulehead += '\n' + (' ' * self.num_indent)
        modulehead += self.clock_name + ', '
        modulehead += self.reset_name + ', '
        modulehead += self.enable_name + ', '
        modulehead += '\n' + (' ' * self.num_indent)
        
        for tk, tv in terms.items():
            scope = util.getScope(tk)
            if util.isTopmodule(scope):
                termtype = self.getTermtype(tk)
                if signaltype.isInput(termtype) and tk == util.toTermname((self.topmodule, self.reset_name)): continue
                if signaltype.isInput(termtype) and tk == util.toTermname((self.topmodule, self.clock_name)): continue
                if signaltype.isInput(termtype): modulehead += util.toFlatname(tk) + ', '
                elif signaltype.isInout(termtype): modulehead += util.toFlatname(tk) + ', '
                elif signaltype.isOutput(termtype): modulehead += util.toFlatname(tk) + ', '
        modulehead = modulehead[:-2] + ');\n\n'
        return modulehead

    ############################################################################
    def _system_io(self, clock_name, reset_name, enable_name):
        flat_clock_name = util.toFlatname( util.toTermname((self.topmodule, clock_name)) )
        flat_reset_name = util.toFlatname( util.toTermname((self.topmodule, reset_name)) )
        flat_enable_name = util.toFlatname( util.toTermname((self.topmodule, enable_name)) )
        code = ''
        code += self._input(clock_name)
        code += self._input(reset_name)
        code += self._input(enable_name)
        code += self._wire(flat_clock_name)
        code += self._wire(flat_reset_name)
        code += self._assign(flat_clock_name, clock_name)
        code += self._assign(flat_reset_name, reset_name)
        code += '\n'
        return code

    ############################################################################
    def _assign(self, left, right, msb=None, lsb=None):
        code = 'assign ' + left 
        if msb is not None and lsb is not None:
            code += '[' + msb.tocode(None)  + ':' + lsb.tocode(None) + ']'
        elif msb is not None:
            code += '[' + msb.tocode(None)  + ']'
        elif lsb is not None:
            code += '[' + lsb.tocode(None)  + ']'
        code += ' = ' + right + ';\n'
        return code

    ############################################################################
    def generateCode(self, targets=()):
        if len(targets) > 0:
            return self.generateSubsetCode(targets)
        return self.generateEntireCode()

    def generateSubsetCode(self, targets):
        terms, parameter, assign, always_clockedge, always_combination = self.getSubset(targets)
        return self._toCode(terms, parameter, assign, always_clockedge, always_combination)

    def generateEntireCode(self):
        terms, parameter, assign, always_clockedge, always_combination = self.getEntire()
        return self._toCode(terms, parameter, assign, always_clockedge, always_combination)

    def _toCode(self, terms, parameter, assign, always_clockedge, always_combination):
        # module header
        modulehead = self._modulehead(terms)

        code = ''
        # clock, reset, control input definition
        code += self._system_io(self.clock_name, self.reset_name, self.enable_name)

        # general signal definition
        for tk, tv in terms.items():
            termtype = self.getTermtype(tk)
            if signaltype.isInput(termtype) and tk == util.toTermname((self.topmodule, self.reset_name)): continue
            if signaltype.isInput(termtype) and tk == util.toTermname((self.topmodule, self.clock_name)): continue
            code += tv.tocode()

        for pk, pv in parameter.items():
            code += pv.tocode()

        # assign
        for ak, avv in assign.items():
            cnt = 0
            for av in avv:
                code += av.tocode()

        # always (clock edge)
        for ck, cvv in always_clockedge.items():
            for cv in cvv:
                code += cv.tocode()

        # always (combination)
        for ck, cvv in always_combination.items():
            for cv in cvv:
                code += cv.tocode()
            
        # module tail
        moduletail = '\nendmodule\n'

        ret = modulehead + code + moduletail
        
        if self.flat:
            ret = re.sub('\.', '_', ret)

        return ret



#-------------------------------------------------------------------------------
# signalvisitor.py
#
# Signal definition visitor
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------
from __future__ import absolute_import
from __future__ import print_function
import sys
import os

from pyverilog.vparser.ast import *
import pyverilog.utils.util as util
import pyverilog.utils.verror as verror
from pyverilog.utils.scope import ScopeLabel, ScopeChain
from pyverilog.dataflow.dataflow import *
from pyverilog.dataflow.visit import *
from pyverilog.dataflow.optimizer import VerilogOptimizer
import pyverilog.dataflow.reorder as reorder

class SignalVisitor(NodeVisitor):
    def __init__(self, moduleinfotable, top):
        self.moduleinfotable = moduleinfotable
        self.top = top
        self.frames = FrameTable()
        self.labels = Labels()
        self.optimizer = VerilogOptimizer({}, {})

        # set the top frame of top module
        self.stackInstanceFrame(top, top)

    ################################################################################
    def getFrameTable(self):
        return self.frames

    ################################################################################
    def start_visit(self):
        return self.visit(self.moduleinfotable.getDefinition(self.top))

    def visit_Input(self, node):
        self.frames.addSignal(node)

    def visit_Output(self, node):
        self.frames.addSignal(node)

    def visit_Inout(self, node):
        self.frames.addSignal(node)

    def visit_Reg(self, node):
        self.frames.addSignal(node)

    def visit_Wire(self, node):
        self.frames.addSignal(node)

    def visit_Supply(self, node):
        self.frames.addSignal(node)

    def visit_RegArray(self, node):
        self.frames.addSignal(node)

    def visit_WireArray(self, node):
        self.frames.addSignal(node)

    def visit_Tri(self, node):
        self.frames.addSignal(node)

    def visit_Integer(self, node):
        self.frames.addSignal(node)

    def visit_Parameter(self, node):
        self.frames.addConst(node)
        name = self.frames.getCurrent() + ScopeLabel(node.name, 'signal')
        if not self.hasConstant(name):
            value = self.optimize(self.getTree(node.value, self.frames.getCurrent()))
            self.setConstant(name, value)

    def visit_Localparam(self, node):
        self.frames.addConst(node)
        name = self.frames.getCurrent() + ScopeLabel(node.name, 'signal')
        if not self.hasConstant(name):
            value = self.optimize(self.getTree(node.value, self.frames.getCurrent()))
            self.setConstant(name, value)

    def visit_Genvar(self, node):
        self.frames.addConst(node)
        name = self.frames.getCurrent() + ScopeLabel(node.name, 'signal')
        if not self.hasConstant(name):
            value = DFEvalValue(0)
            self.setConstant(name, value)

    def visit_Function(self, node):
        self.frames.addFunction(node)
        self.frames.setFunctionDef()
        self.generic_visit(node)
        self.frames.unsetFunctionDef()

    def visit_Task(self, node):
        self.frames.addTask(node)
        self.frames.setTaskDef()
        self.generic_visit(node)
        self.frames.unsetTaskDef()

    def visit_Initial(self, node):
        pass
        #label = self.labels.get( self.frames.getLabelKey('initial') )
        #current = self.frames.addFrame(ScopeLabel(label, 'initial'),
        #                               generate=self.frames.isGenerate(),
        #                               initial=True)
        #self.generic_visit(node)
        #self.frames.setCurrent(current)

    def visit_InstanceList(self, node):
        for i in node.instances:
            self.visit(i)

    def visit_Instance(self, node):
        if node.array: return self._visit_Instance_array(node)
        nodename = node.name
        return self._visit_Instance_body(node, nodename)

    def _visit_Instance_array(self, node):
        if node.name == '':
            raise verror.FormatError("Module %s requires an instance name" % node.module)

        current = self.frames.getCurrent()
        msb = self.optimize(self.getTree(node.array.msb, current)).value
        lsb = self.optimize(self.getTree(node.array.lsb, current)).value

        for i in range(lsb, msb+1):
            nodename = node.name + '_' + str(i)
            self._visit_Instance_body(node, nodename)

    def _visit_Instance_body(self, node, nodename):
        if node.module in primitives: return self._visit_Instance_primitive(node)

        if nodename == '':
            raise verror.FormatError("Module %s requires an instance name" % node.module)

        current = self.stackInstanceFrame(nodename, node.module)

        self.setInstanceSimpleConstantTerms()

        scope = self.frames.getCurrent()

        paramnames = self.moduleinfotable.getParamNames(node.module)
        for paramnames_i, param in enumerate(node.parameterlist):
            paramname = paramnames[paramnames_i] if param.paramname is None else param.paramname
            if paramname not in paramnames:
                raise verror.FormatError("No such parameter: %s in %s" %
                                         (paramname, nodename))
            value = self.optimize(self.getTree(param.argname, current))
            name, definition = self.searchConstantDefinition(scope, paramname)
            self.setConstant(name, value)

        self.setInstanceConstants()
        self.setInstanceConstantTerms()

        self.visit(self.moduleinfotable.getDefinition(node.module))
        self.frames.setCurrent(current)

    def _visit_Instance_primitive(self, node):
        pass

    def visit_Always(self, node):
        label = self.labels.get( self.frames.getLabelKey('always') )
        current = self.frames.addFrame(ScopeLabel(label, 'always'),
                                       generate=self.frames.isGenerate(),
                                       always=True)
        self.generic_visit(node)
        self.frames.setCurrent(current)

    def visit_IfStatement(self, node):

        if (self.frames.isGenerate() and
            not self.frames.isAlways() and not self.frames.isInitial() and
            not self.frames.isFunctioncall() and not self.frames.isTaskcall() and
            not self.frames.isFunctiondef() and not self.frames.isTaskdef()):
            # generate-if statement
            current = self.frames.getCurrent()
            tree = self.getTree(node.cond, current)
            rslt = self.optimize(tree)
            if not isinstance(rslt, DFEvalValue):
                raise verror.FormatError("Can not resolve generate-if condition")
            if rslt.value > 0:
                label = self._if_true(node)
            else:
                label = self.labels.get( self.frames.getLabelKey('if') )
                self._if_false(node, label)
            return

        label = self._if_true(node)
        self._if_false(node, label)

    def _toELSE(self, label):
        return label + '_ELSE'

    def _if_true(self, node):
        if node.true_statement is None: return None
        label = self.labels.get( self.frames.getLabelKey('if') )
        current = self.frames.addFrame(ScopeLabel(label, 'if'),
                                       frametype='ifthen',
                                       condition=node.cond,
                                       functioncall=self.frames.isFunctioncall(),
                                       taskcall=self.frames.isTaskcall(),
                                       generate=self.frames.isGenerate(),
                                       always=self.frames.isAlways(),
                                       initial=self.frames.isInitial())
        self.visit(node.true_statement)
        self.frames.setCurrent(current)
        return label

    def _if_false(self, node, label):
        if node.false_statement is None: return
        label = self._toELSE(label)
        current = self.frames.addFrame(ScopeLabel(label, 'if'),
                                       frametype='ifelse',
                                       condition=node.cond,
                                       functioncall=self.frames.isFunctioncall(),
                                       taskcall=self.frames.isTaskcall(),
                                       generate=self.frames.isGenerate(),
                                       always=self.frames.isAlways(),
                                       initial=self.frames.isInitial())
        self.visit(node.false_statement)
        self.frames.setCurrent(current)
        return label

    def visit_CaseStatement(self, node):
        start_frame = self.frames.getCurrent()
        self._case(node.comp, node.caselist)
        self.frames.setCurrent(start_frame)

    def visit_CasexStatement(self, node):
        return self.visit_CaseStatement(node)
        
    def _case(self, comp, caselist):
        if len(caselist) == 0: return
        case = caselist[0]
        cond = IntConst('1')
        if case.cond is not None:
            if len(case.cond) > 1:
                cond = Eq(comp, case.cond[0])
                for c in case.cond[1:]:
                    cond = Lor(cond, Eq(comp, c))
            else:
                cond = Eq(comp, case.cond[0])
        label = self.labels.get( self.frames.getLabelKey('if') )
        current = self.frames.addFrame(ScopeLabel(label, 'if'),
                                       frametype='ifthen',
                                       condition=cond,
                                       functioncall=self.frames.isFunctioncall(),
                                       taskcall=self.frames.isTaskcall(),
                                       generate=self.frames.isGenerate(),
                                       always=self.frames.isAlways(),
                                       initial=self.frames.isInitial())
        if case.statement is not None: self.visit(case.statement)
        self.frames.setCurrent(current)
        if len(caselist) == 1: return
        label = self._toELSE(label)
        current = self.frames.addFrame(ScopeLabel(label, 'if'),
                                       frametype='ifelse',
                                       condition=cond,
                                       functioncall=self.frames.isFunctioncall(),
                                       taskcall=self.frames.isTaskcall(),
                                       generate=self.frames.isGenerate(),
                                       always=self.frames.isAlways(),
                                       initial=self.frames.isInitial())
        self._case(comp, caselist[1:])

    def visit_ForStatement(self, node):
        ## pre-statement
        current = self.frames.getCurrent()
        pre_right = self.getTree(node.pre.right, current)
        pre_right_value = self.optimize(pre_right)
        loop = pre_right_value.value
        self.frames.setForPre()
        self.visit(node.pre)
        self.frames.unsetForPre()
        label = self.labels.get( self.frames.getLabelKey('for') )
        #loop = 0
        while True:
            ## cond-statement
            current = self.frames.getCurrent()
            tree = self.getTree(node.cond, current)
            rslt = self.optimize(tree)
            if not isinstance(rslt, DFEvalValue):
                raise verror.FormatError(("Can not process the for-statement. "
                                          "for-condition should be evaluated statically."))
            # loop termination
            if rslt.value <= 0: break

            ## main-statement
            current = self.frames.addFrame(ScopeLabel(label, 'for', loop),
                                           frametype='for',
                                           functioncall=self.frames.isFunctioncall(),
                                           taskcall=self.frames.isTaskcall(),
                                           generate=self.frames.isGenerate(),
                                           always=self.frames.isAlways(),
                                           initial=self.frames.isInitial(),
                                           loop=loop, loop_iter=self.frames.getForIter())
            self.visit(node.statement)
            self.frames.setCurrent(current)

            ## post-statement
            current = self.frames.getCurrent()
            post_right = self.getTree(node.post.right, current)
            post_right_value = self.optimize(post_right)
            loop = post_right_value.value
            self.frames.setForPost()
            self.visit(node.post)
            self.frames.unsetForPost()
            #loop += 1

    def visit_WhileStatement(self, node):
        pass

    def visit_GenerateStatement(self, node):
        label = self.labels.get( self.frames.getLabelKey('generate') )
        current = self.frames.addFrame(ScopeLabel(label, 'generate'),
                                       generate=True)
        self.generic_visit(node)
        self.frames.setCurrent(current)

    def visit_Block(self, node):
        label = None
        if node.scope is not None:
            label = node.scope
        else:
            label = self.labels.get( self.frames.getLabelKey('block') )
        current = self.frames.addFrame(ScopeLabel(label, 'block'),
                                       frametype='block',
                                       functioncall=self.frames.isFunctioncall(),
                                       taskcall=self.frames.isTaskcall(),
                                       generate=self.frames.isGenerate(),
                                       always=self.frames.isAlways(),
                                       initial=self.frames.isInitial())
        self.generic_visit(node)
        self.frames.setCurrent(current)

    def visit_Assign(self, node):
        pass

    def visit_BlockingSubstitution(self, node):
        if self.frames.isForpre() or self.frames.isForpost():
            current = self.frames.getCurrent()
            name, definition = self.searchConstantDefinition(current,
                                                             node.left.var.name)
            value = self.optimize(self.getTree(node.right.var, current))
            self.setConstant(name, value)
            self.frames.setForIter(name)

    def visit_NonblockingSubstitution(self, node):
        pass

    ############################################################################
    def optimize(self, node):
        return self.optimizer.optimize(node)

    def stackInstanceFrame(self, instname, modulename):
        current = self.frames.addFrame(ScopeLabel(instname, 'module'), module=True,
                                       modulename=modulename)
        self.frames.updateSignal(self.moduleinfotable.getSignals(modulename))
        self.frames.updateConst(self.moduleinfotable.getConsts(modulename))
        return current

    def setInstanceSimpleConstantTerms(self):
        current = self.frames.getCurrent()
        for name, definitions in self.frames.getConsts(current).items():
            if len(definitions) > 1:
                raise verror.FormatError("Multiple definitions for Constant")
            for definition in definitions:
                simple_definition = copy.deepcopy(definition)
                if simple_definition.width is not None:
                    simple_definition.width.msb = None
                    simple_definition.width.lsb = None
                term = self.makeConstantTerm(name, simple_definition, current)
                self.setConstantTerm(name, term)

    def setInstanceConstantTerms(self):
        current = self.frames.getCurrent()
        for name, definitions in self.frames.getConsts(current).items():
            if len(definitions) > 1:
                raise verror.FormatError("Multiple definitions for Constant")
            for definition in definitions:
                term = self.makeConstantTerm(name, definition, current)
                self.setConstantTerm(name, term)

    def setInstanceConstants(self):
        current = self.frames.getCurrent()

        all_passed = False
        while not all_passed:
            all_passed = True
            for name, definitions in self.frames.getConsts(current).items():
                if len(definitions) > 1:
                    raise verror.FormatError("Multiple definitions for Constant")

                if self.hasConstant(name):
                    continue

                for definition in definitions:
                    if isinstance(definition, Genvar):
                        continue
                    value = self.optimize(self.getTree(definition.value, current))
                    if not isinstance(value, DFEvalValue):
                        all_passed = False
                        continue

                    self.setConstant(name, value)

    def setConstant(self, name, value):
        self.optimizer.setConstant(name, value)

    def getConstant(self, name):
        return self.optimizer.getConstant(name)

    def hasConstant(self, name):
        return self.optimizer.hasConstant(name)

    def setConstantTerm(self, name, term):
        self.optimizer.setTerm(name, term)

    def hasConstantTerm(self, name):
        self.optimizer.hasTerm(name)

    ############################################################################
    def toScopeChain(self, blocklabel):
        scopelist = []
        for b in blocklabel.labellist:
            if b.loop is not None:
                loop = self.optimize(b.loop)
                if not isinstance(loop, DFEvalValue):
                    raise verror.FormatError('Loop iterator should be constant')
                scopelist.append( ScopeLabel(b.name, 'for', loop) )
            scopelsit.append( ScopeLabel(b.name, 'any') )
        return ScopeChain( scopelist )

    def searchScopeConstantValue(self, blocklabel, name):
        currentmodule = self.frames.getCurrentModuleScopeChain()
        localchain = currentmodule[-1:] + self.toScopeChain(blocklabel)
        matchedchain = self.frames.searchMatchedScopeChain(currentmodule, localchain)
        varname = currentmodule[:-1] + matchedchain + ScopeLabel(name, 'signal')
        const = self.getConstant(varname)
        return const

    #def searchScopeConstantDefinition(self, blocklabel, name):
    #    currentmodule = self.frames.getCurrentModuleScopeChain()
    #    localchain = currentmodule[-1:] + self.toScopeChain(blocklabel)
    #    matchedchain = self.frames.searchMatchedScopeChain(currentmodule, localchain)
    #    constdef = self.frames.getConstantDefinition(matchedchain, name)
    #    return constdef

    def searchConstantDefinition(self, key, name):
        foundkey, founddef = self.frames.searchConstantDefinition(key, name)
        if foundkey is not None:
            return foundkey + ScopeLabel(name, 'signal'), founddef
        foundkey, founddef = self.frames.searchSignalDefinition(key, name)
        if foundkey is not None:
            return foundkey + ScopeLabel(name, 'signal'), founddef
        if foundkey is None:
            raise verror.DefinitionError('constant value not found: %s' % name)

    def searchConstantValue(self, key, name):
        foundkey, founddef = self.searchConstantDefinition(key, name)
        const = self.getConstant(foundkey)
        return const

    ############################################################################
    def makeConstantTerm(self, name, node, scope):
        termtype = node.__class__.__name__
        termtypes = set([termtype])
        msb = DFIntConst('31') if node.width is None else self.makeDFTree(node.width.msb, scope)
        lsb = DFIntConst('0') if node.width is None else self.makeDFTree(node.width.lsb, scope)
        return Term(name, termtypes, msb, lsb)

    def getTree(self, node, scope):
        expr = node.var if isinstance(node, Rvalue) else node
        return self.makeDFTree(expr, scope)

    def makeDFTree(self, node, scope):
        if isinstance(node, str):
            return self.searchConstantValue(scope, node)

        if isinstance(node, Identifier):
            if node.scope is not None:
                const = self.searchScopeConstantValue(node.scope, node.name)
                return const
            return self.searchConstantValue(scope, node.name)

        if isinstance(node, IntConst):
            return DFIntConst(node.value)

        if isinstance(node, FloatConst):
            return DFFloatConst(node.value)

        if isinstance(node, StringConst):
            return DFStringConst(node.value)

        if isinstance(node, Cond):
            true_df = self.makeDFTree(node.true_value, scope)
            false_df = self.makeDFTree(node.false_value, scope)
            cond_df = self.makeDFTree(node.cond, scope)
            if isinstance(cond_df, DFBranch):
                return reorder.insertCond(cond_df, true_df, false_df)
            return DFBranch(cond_df, true_df, false_df)

        if isinstance(node, UnaryOperator):
            right_df = self.makeDFTree(node.right, scope)
            if isinstance(right_df, DFBranch):
                return reorder.insertUnaryOp(right_df, node.__class__.__name__)
            return DFOperator((right_df,), node.__class__.__name__)

        if isinstance(node, Operator):
            left_df = self.makeDFTree(node.left, scope)
            right_df = self.makeDFTree(node.right, scope)
            if isinstance(left_df, DFBranch) or isinstance(right_df, DFBranch):
                return reorder.insertOp(left_df, right_df, node.__class__.__name__)
            return DFOperator((left_df, right_df,), node.__class__.__name__)

        if isinstance(node, Partselect):
            var_df = self.makeDFTree(node.var, scope)
            msb_df = self.makeDFTree(node.msb, scope)
            lsb_df = self.makeDFTree(node.lsb, scope)

            if isinstance(var_df, DFBranch):
                return reorder.insertPartselect(var_df, msb_df, lsb_df)
            return DFPartselect(var_df, msb_df, lsb_df)

        if isinstance(node, Pointer):
            var_df = self.makeDFTree(node.var, scope)
            ptr_df = self.makeDFTree(node.ptr, scope)

            if (isinstance(var_df, DFTerminal) and
                (signaltype.isRegArray(self.getTermtype(var_df.name)) or
                 signaltype.isWireArray(self.getTermtype(var_df.name)))):
                return DFPointer(var_df, ptr_df)
            return DFPartselect(var_df, ptr_df, copy.deepcopy(ptr_df))

        if isinstance(node, Concat):
            nextnodes = []
            for n in node.list:
                nextnodes.append(self.makeDFTree(n, scope))
            for n in nextnodes:
                if isinstance(n, DFBranch):
                    return reorder.insertConcat(tuple(nextnodes))
            return DFConcat(tuple(nextnodes))

        if isinstance(node, Repeat):
            nextnodes = []
            times = self.optimize(self.getTree(node.times, scope)).value
            value = self.makeDFTree(node.value, scope)
            for i in range(int(times)):
                nextnodes.append(copy.deepcopy(value))
            return DFConcat(tuple(nextnodes))

        if isinstance(node, SystemCall):
            if node.syscall == 'unsigned':
                return self.makeDFTree(node.args[0])
            if node.syscall == 'signed':
                return self.makeDFTree(node.args[0])
            return DFIntConst('0')

        raise verror.FormatError("unsupported AST node type: %s %s" %
                                 (str(type(node)), str(node)))




#-------------------------------------------------------------------------------
# dataflow.py
#
# Basic classes of Data flow nodes
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
# Contributor: ryosuke fukatani
#-------------------------------------------------------------------------------
from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import re
import copy

################################################################################
dfnodelist = ('DFIntConst', 'DFFloatConst', 'DFStringConst',
              'DFEvalValue', 'DFUndefined', 'DFHighImpedance',
              'DFTerminal',
              'DFBranch', 'DFOperator', 'DFPartselect', 'DFPointer',
              'DFConcat', 'DFDelay', 'DFSyscall')

def printIndent(s, indent=4):
    print((' ' * indent) + s)

def generateWalkTree(offset=1):
    base_indent = 4
    printIndent('def walkTree(tree):', base_indent*(0+offset))
    for df in dfnodelist:
        printIndent('if isinstance(tree, %s):' % df, base_indent * (1+offset))
        printIndent('pass', base_indent * (2+offset))

if __name__ == '__main__':
    generateWalkTree()
    exit()

################################################################################
import pyverilog.utils.verror as verror
import pyverilog.utils.util as util
import pyverilog.utils.signaltype as signaltype
import pyverilog.utils.op2mark as op2mark

################################################################################
class DFNode(object):
    attr_names = ()
    def __init__(self): pass
    def __repr__(self): pass
    def tostr(self): pass
    def tocode(self, dest='dest'): return self.__repr__()
    def tolabel(self): return self.__repr__()
    def children(self):
        nodelist = []
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return False
    def __ne__(self, other):
        return not self.__eq__(other)
    def __hash__(self):
        return id(self)

class DFTerminal(DFNode):
    attr_names = ('name',)
    def __init__(self, name):
        self.name = name
    def __repr__(self):
        ret = ''
        for n in self.name:
            ret += str(n) + '.'
        return ret[:-1]
    def tostr(self):
        ret = '(Terminal '
        for n in self.name:
            ret += str(n) + '.'
        return ret[0:-1] + ')'
    def tocode(self, dest='dest'):
        #ret = ''
        #for n in self.name:
        #    ret += n.tocode() + '__'
        #return ret[:-2]
        return self.name.tocode()
    def tolabel(self):
        return self.tocode()
    def children(self):
        nodelist = []
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.name == other.name
    def __hash__(self):
        return hash(self.name)

class DFConstant(DFNode):
    attr_names = ('value',)
    def __init__(self, value):
        self.value = value
    def __repr__(self):
        return str(self.value)
    def tostr(self):
        ret = '(Constant ' + str(self.value) + ')'
        return ret
    def children(self):
        nodelist = []
        return tuple(nodelist)
    def eval(self):
        return None
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.value == other.value
    def __hash__(self):
        return hash(self.value)

class DFIntConst(DFConstant):
    def __init__(self, value):
        self.value = value
    def tostr(self):
        ret = '(IntConst ' + str(self.value) + ')'
        return ret
    def eval(self):
        targ = self.value.replace('_','')
        signed = False
        match = re.search(r's(.+)', targ)
        if match is not None:
            signed = True
        match = re.search(r'h(.+)', targ)
        if match is not None:
            return int(match.group(1), 16)
        match = re.search(r'd(.+)', targ)
        if match is not None:
            return int(match.group(1), 10)
        match = re.search(r'o(.+)', targ)
        if match is not None:
            return int(match.group(1), 8)
        match = re.search(r'b(.+)', targ)
        if match is not None:
            return int(match.group(1), 2)
        return int(targ, 10)
    def width(self):
        targ = self.value.replace('_','')
        match = re.search(r'(.+)\'h.+', targ)
        if match is not None:
            return int(match.group(1), 10)
        match = re.search(r'(.+)\'d.+', targ)
        if match is not None:
            return int(match.group(1), 10)
        match = re.search(r'(.+)\'o.+', targ)
        if match is not None:
            return int(match.group(1), 10)
        match = re.search(r'(.+)\'b.+', targ)
        if match is not None:
            return int(match.group(1), 10)
        return 32

class DFFloatConst(DFConstant):
    def __init__(self, value):
        self.value = value
    def tostr(self):
        ret = '(FloatConst ' + str(self.value) + ')'
        return ret
    def eval(self):
        return float(self.value)

class DFStringConst(DFConstant):
    def __init__(self, value):
        self.value = value
    def tostr(self):
        ret = '(StringConst ' + str(self.value) + ')'
        return ret
    def eval(self):
        return self.value

################################################################################
class DFNotTerminal(DFNode): pass

class DFOperator(DFNotTerminal):
    attr_names = ('operator',)
    def __init__(self, nextnodes, operator):
        self.nextnodes = nextnodes
        self.operator = operator

        for n in nextnodes:
            if n is None: raise verror.DefinitionError()

    def __repr__(self):
        return self.operator
    def tostr(self):
        ret = '(Operator ' + self.operator
        ret += ' Next:'
        for n in self.nextnodes: ret += n.tostr() + ','
        ret = ret[0:-1] + ')'
        return ret
    def tocode(self, dest='dest'):
        ret = ''
        if len(self.nextnodes) > 1:
            ret += '(' + self.nextnodes[0].tocode(dest)
            ret += op2mark.op2mark(self.operator)
            ret += self.nextnodes[1].tocode(dest) + ')'
        else:
            ret += '(' + op2mark.op2mark(self.operator)
            ret += self.nextnodes[0].tocode(dest) + ')'
        return ret
    def children(self):
        nodelist = []
        if self.nextnodes is not None: nodelist.extend(self.nextnodes)
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.operator == other.operator and self.nextnodes == other.nextnodes
    def __hash__(self):
        return hash((self.operator, tuple(self.nextnodes)))

class DFPartselect(DFNotTerminal):
    attr_names = ()
    def __init__(self, var, msb, lsb):
        self.var = var
        self.msb = msb
        self.lsb = lsb
    def __repr__(self):
        return 'PartSelect'
    def tostr(self):
        ret = '(Partselect'
        ret += ' Var:' + self.var.tostr()
        ret += ' MSB:' + self.msb.tostr()
        ret += ' LSB:' + self.lsb.tostr()
        ret += ')'
        return ret
    def tocode(self, dest='dest'):
        ret = self.var.tocode(dest)
        msbcode = self.msb.tocode(dest)
        lsbcode = self.lsb.tocode(dest)
        if msbcode == lsbcode:
            ret += '[' + msbcode + ']'
        else:
            ret += '[' + msbcode
            ret += ':' + lsbcode + ']'
        return ret
    def children(self):
        nodelist = []
        if self.var is not None: nodelist.append(self.var)
        if self.msb is not None: nodelist.append(self.msb)
        if self.lsb is not None: nodelist.append(self.lsb)
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.var == other.var and self.msb == other.msb and self.lsb == other.lsb
    def __hash__(self):
        return hash((self.var, self.msb, self.lsb))

class DFPointer(DFNotTerminal):
    attr_names = ()
    def __init__(self, var, ptr):
        self.var = var
        self.ptr = ptr
    def __repr__(self):
        return 'Pointer'
    def tostr(self):
        ret = '(Pointer'
        ret += ' Var:' + self.var.tostr()
        ret += ' PTR:' + self.ptr.tostr()
        ret += ')'
        return ret
    def tocode(self, dest='dest'):
        ret = self.var.tocode(dest)
        ret += '[' + self.ptr.tocode(dest) + ']'
        return ret
    def children(self):
        nodelist = []
        if self.var is not None: nodelist.append(self.var)
        if self.ptr is not None: nodelist.append(self.ptr)
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.var == other.var and self.ptr == other.ptr
    def __hash__(self):
        return hash((self.var, self.ptr))

class DFConcat(DFNotTerminal):
    attr_names = ()
    def __init__(self, nextnodes):
        self.nextnodes = nextnodes
    def __repr__(self):
        return 'Concat'
    def tostr(self):
        ret = '(Concat'
        ret += ' Next:'
        for n in self.nextnodes: ret += n.tostr() + ','
        ret = ret[0:-1] + ')'
        return ret
    def tocode(self, dest='dest'):
        ret = '{'
        for n in self.nextnodes: ret += n.tocode(dest) + ','
        ret = ret[:-1]
        ret += '}'
        return ret
    def children(self):
        nodelist = []
        if self.nextnodes is not None: nodelist.extend(self.nextnodes)
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.nextnodes == other.nextnodes
    def __hash__(self):
        return hash(tuple(self.nextnodes))

################################################################################
class DFBranch(DFNotTerminal):
    attr_names = ()
    def __init__(self, condnode, truenode, falsenode):
        self.condnode = condnode
        self.truenode = truenode
        self.falsenode = falsenode
    def __repr__(self):
        return 'Branch'
    def tostr(self):
        ret = '(Branch'
        if self.condnode is not None: ret += ' Cond:' + self.condnode.tostr()
        if self.truenode is not None: ret += ' True:' + self.truenode.tostr()
        if self.falsenode is not None: ret += ' False:'+ self.falsenode.tostr()
        ret += ')'
        return ret
    def tocode(self, dest='dest', always=''):
        if always == 'clockedge': return self._tocode_always(dest, always)
        if always == 'combination': return self._tocode_always(dest, always)
        ret = '('
        if self.condnode is not None: ret += '(' + self.condnode.tocode(dest) + ')'
        ret += '? '
        if self.truenode is not None: ret += self.truenode.tocode(dest)
        else: ret += dest
        ret += " : "
        if self.falsenode is not None: ret += self.falsenode.tocode(dest)
        else: ret += dest
        ret += ")"
        return ret
    def _tocode_always(self, dest='dest', always='clockedge'):
        ret = 'if('
        if self.condnode is not None: ret += self.condnode.tocode(dest)
        ret += ') begin\n'
        if self.truenode is not None:
            if isinstance(self.truenode, DFBranch):
                ret += self.truenode.tocode(dest, always=always)
            elif always == 'clockedge':
                ret += dest + ' <= ' + self.truenode.tocode(dest) + ';\n'
            elif always == 'combination':
                ret += dest + ' = ' + self.truenode.tocode(dest) + ';\n'
        ret += 'end\n'
        if self.falsenode is not None:
            ret += 'else begin\n'
            if isinstance(self.falsenode, DFBranch):
                ret += self.falsenode.tocode(dest, always=always)
            elif always == 'clockedge':
                ret += dest + ' <= ' + self.falsenode.tocode(dest) + ';\n'
            elif always == 'combination':
                ret += dest + ' = ' + self.falsenode.tocode(dest) + ';\n'
            ret += 'end\n'
        return ret
    def children(self):
        nodelist = []
        if self.truenode is not None: nodelist.append(self.truenode)
        if self.condnode is not None: nodelist.append(self.condnode)
        if self.falsenode is not None: nodelist.append(self.falsenode)
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.condnode == other.condnode and self.truenode == other.truenode and self.falsenode == other.falsenode
    def __hash__(self):
        return hash((self.condnode, self.truenode, self.falsenode))

################################################################################
class DFEvalValue(DFNode):
    attr_names = ('value', 'width',)
    def __init__(self, value, width=32, isfloat=False, isstring=False):
        self.value = value
        self.width = width
        self.isfloat = isfloat
        self.isstring = isstring
    def __repr__(self):
        if self.isstring:
            return self.value
        ret = ''
        if self.value < 0: ret += '(-'
        if self.width != 32: ret += str(self.width)
        ret += "'d"
        ret += str(abs(self.value))
        if self.value < 0: ret += ')'
        return ret
    def tostr(self):
        if self.isstring:
            return self.value
        ret = ''
        if self.value < 0: ret += '(-'
        if self.width != 32: ret += str(self.width)
        ret += "'d"
        ret += str(abs(self.value))
        if self.value < 0: ret += ')'
        return ret
    def tocode(self, dest='dest'):
        return self.__repr__()
    def children(self):
        nodelist = []
        return tuple(nodelist)
    def eval(self):
        return self.value
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.value == other.value and self.width == other.width and self.isfloat == other.isfloat and self.isstring == other.isstring
    def __hash__(self):
        return hash((self.value, self.width, self.isfloat, self.isstring))

class DFUndefined(DFNode):
    attr_names = ('width',)
    def __init__(self, width):
        self.width = width
    def __repr__(self):
        ret = ''
        if self.width != 32: ret += str(self.width)
        ret += "'d"
        ret += 'x'
        return ret
    def tostr(self):
        ret = ''
        if self.width != 32: ret += str(self.width)
        ret += "'d"
        ret += 'x'
        return ret
    def children(self):
        nodelist = []
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.width == other.width
    def __hash__(self):
        return hash(self.width)

class DFHighImpedance(DFNode):
    attr_names = ('width',)
    def __init__(self, width):
        self.width = width
    def __repr__(self):
        ret = ''
        if self.width != 32: ret += str(self.width)
        ret += "'d"
        ret += 'z'
        return ret
    def tostr(self):
        ret = ''
        if self.width != 32: ret += str(self.width)
        ret += "'d"
        ret += 'z'
        return ret
    def children(self):
        nodelist = []
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.width == other.width
    def __hash__(self):
        return hash(self.width)

################################################################################
class DFDelay(DFNotTerminal):
    attr_names = ()
    def __init__(self, nextnode):
        self.nextnode = nextnode
    def __repr__(self):
        return 'Delay'
    def tostr(self):
        ret = '(Delay '
        if self.nextnode is not None: ret += self.nextnode.tostr()
        ret += ')'
        return ret
    def tocode(self, dest='dest'):
        raise verror.DefinitionError('DFDelay does not support tocode()')
    def children(self):
        nodelist = []
        if self.nextnode is not None: nodelist.append(self.nextnode)
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.nextnodes == other.nextnodes
    def __hash__(self):
        return hash(tuple(self.nextnodes))

################################################################################
class DFSyscall(DFNotTerminal):
    attr_names = ()
    def __init__(self, syscall, nextnodes):
        self.syscall = syscall
        self.nextnodes = nextnodes
    def __repr__(self):
        return 'Syscall'
    def tostr(self):
        ret = '(Syscall '
        ret += self.syscall
        ret += ' Next:'
        for n in self.nextnodes: ret += n.tostr() + ','
        ret = ret[0:-1] + ')'
        return ret
    def tocode(self, dest='dest'):
        ret = '$' + self.syscall + '('
        for n in self.nextnodes: ret += n.tocode(dest) + ','
        ret = ret[:-1]
        ret += ')'
        return ret
    def children(self):
        nodelist = []
        if self.nextnodes is not None: nodelist.extend(self.nextnodes)
        return tuple(nodelist)
    def __eq__(self, other):
        if type(self) != type(other): return False
        if self.syscall != other.syscall: return False
        return self.nextnodes == other.nextnodes
    def __hash__(self):
        return hash(tuple(self.nextnodes))

################################################################################
class Term(object):
    def __init__(self,name,termtype=(),msb=None,lsb=None,lenmsb=None,lenlsb=None):
        self.name = name # tuple (str)
        self.termtype = termtype # set (str)
        self.msb = msb # DFNode
        self.lsb = lsb # DFNode
        self.lenmsb = lenmsb # DFNode
        self.lenlsb = lenlsb # DFNode
    def __repr__(self):
        return str(self.name)
    def tostr(self):
        ret = '(Term name:' + str(self.name) + ' type:' + str(sorted(self.termtype, key=lambda x:str(x)))
        if self.msb is not None: ret += ' msb:' + self.msb.tostr()
        if self.lsb is not None: ret += ' lsb:' + self.lsb.tostr()
        if self.lenmsb is not None: ret += ' lenmsb:' + self.lenmsb.tostr()
        if self.lenlsb is not None: ret += ' lenlsb:' + self.lenlsb.tostr()
        ret += ')'
        return ret
    def __ne__(self, other):
        return not self.__eq__(other)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.name == other.name and self.termtype == other.termtype and self.msb == other.msb and self.lsb == other.lsb and self.lenmsb == other.lenmsb and self.lenlsb == other.lenlsb
    def __hash__(self):
        return hash((self.name, self.termtype, self.msb, self.lsb, self.lenmsb, self.lenlsb))

    ############################################################################
    def getScope(self, termname):
        return termname[:-1]

    def isTopmodule(self, scope):
        if len(scope)==1: return True
        return False

    ############################################################################
    def tocode(self):
        flatname = util.toFlatname(self.name)
        scope = self.getScope(self.name)
        code = ''
        if self.isTopmodule(scope):
            if signaltype.isInput(self.termtype): code += 'input '
            elif signaltype.isInout(self.termtype): code += 'inout '
            elif signaltype.isOutput(self.termtype): code += 'output '
        else:
            if signaltype.isInput(self.termtype): code += 'wire '
            elif signaltype.isInout(self.termtype): code += 'wire '
            elif signaltype.isOutput(self.termtype) and not signaltype.isReg(self.termtype): code += 'wire '

        if signaltype.isReg(self.termtype): code += 'reg '
        if signaltype.isRegArray(self.termtype): code += 'reg '
        if signaltype.isWire(self.termtype): code += 'wire '
        if signaltype.isWireArray(self.termtype): code += 'wire '
        if signaltype.isInteger(self.termtype): code += 'integer '
        if signaltype.isFunction(self.termtype): code += 'wire '
        if signaltype.isRename(self.termtype): code += 'wire '

        if (not signaltype.isInteger(self.termtype) and
            self.msb is not None and self.lsb is not None):
            code += '[' + self.msb.tocode(None) + ':' + self.lsb.tocode(None) + '] '
        code += flatname # signal name
        if self.lenmsb is not None and self.lenlsb is not None:
            code += ' [' + self.lenmsb.tocode() + ':' + self.lenlsb.tocode(flatname) + ']'
        code += ';\n'
        return code

################################################################################
class Bind(object):
    def __init__(self, tree, dest, msb=None, lsb=None, ptr=None, alwaysinfo=None, parameterinfo=''):
        self.tree = tree
        self.dest = dest
        self.msb = msb
        self.lsb = lsb
        self.ptr = ptr
        self.alwaysinfo = alwaysinfo
        self.parameterinfo = parameterinfo
        if dest is None: raise verror.DefinitionError('Bind dest is empty')

    def __ne__(self, other):
        return not self.__eq__(other)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.tree == other.tree and self.dest == other.dest and self.msb == other.msb and self.lsb == other.lsb and self.ptr == other.ptr and self.alwaysinfo == other.alwaysinfo and self.parameterinfo == other.parameterinfo
    def __hash__(self):
        return hash((self.tree, self.dest, self.msb, self.lsb, self.ptr, self.alwaysinfo, self.parameterinfo))

    def isCombination(self):
        if self.alwaysinfo is None: return True
        if self.alwaysinfo.isCombination(): return True
        return False

    def tostr(self):
        ret = '(Bind'
        if self.dest is not None: ret += ' dest:' + str(self.dest)
        if self.msb is not None: ret += ' msb:' + self.msb.tostr()
        if self.lsb is not None: ret += ' lsb:' + self.lsb.tostr()
        if self.ptr is not None: ret += ' ptr:' + self.ptr.tostr()
        if self.tree is not None: ret += ' tree:' + self.tree.tostr()
        ret += ')'
        return ret

    def tocode(self):
        if self.parameterinfo == 'parameter': return self._parameter()
        if self.parameterinfo == 'localparam': return self._localparam()
        if self.alwaysinfo is None: return self._assign()
        if self.alwaysinfo.isCombination(): return self._always_combination()
        else: return self._always_clockedge()

    def getdest(self):
        dest = util.toFlatname(self.dest)
        if self.ptr is not None:
            dest += '[' + self.ptr.tocode(None) + ']'
        if self.msb is not None and self.lsb is not None:
            msbcode = self.msb.tocode(None)
            lsbcode = self.lsb.tocode(None)
            if msbcode == lsbcode:
                dest += '[' + msbcode + ']'
            else:
                dest += '[' + msbcode + ':' + lsbcode + ']'
        return dest

    def _parameter(self):
        dest = self.getdest()
        code = 'parameter ' + dest
        code += ' = ' + self.tree.tocode(dest) + ';\n'
        return code

    def _localparam(self):
        dest = self.getdest()
        code = 'localparam ' + dest
        code += ' = ' + self.tree.tocode(dest) + ';\n'
        return code

    def _assign(self):
        dest = self.getdest()
        code = 'assign ' + dest
        code += ' = ' + self.tree.tocode(dest) + ';\n'
        return code

    def _always_clockedge(self):
        dest = self.getdest()
        code = 'always @('
        if self.alwaysinfo.clock_edge is not None and self.alwaysinfo.clock_name is not None:
            code += self.alwaysinfo.clock_edge + ' '
            code += util.toFlatname(self.alwaysinfo.clock_name)
        if self.alwaysinfo.reset_edge is not None and self.alwaysinfo.reset_name is not None:
            code += ' or '
            code += self.alwaysinfo.reset_edge + ' '
            code += util.toFlatname(self.alwaysinfo.reset_name)
        code += ') begin\n'
        if isinstance(self.tree, DFBranch):
            code += self.tree.tocode(dest, always='clockedge')
        else:
            code += dest
            code += ' <= ' + self.tree.tocode(dest) + ';\n'
        code += 'end\n'
        code += '\n'
        return code

    def _always_combination(self):
        dest = self.getdest()
        code = ''
        code += 'always @*'
        code += ' begin\n'
        if isinstance(self.tree, DFBranch):
            code += self.tree.tocode(dest, always='combination')
        else:
            code += dest
            code += ' = ' + self.tree.tocode(dest) + ';\n'
        code += 'end\n'
        code += '\n'
        return code

    def isClockEdge(self):
        if self.alwaysinfo is None: return False
        return self.alwaysinfo.isClockEdge()
    def getClockName(self):
        if self.alwaysinfo is None: return ''
        return self.alwaysinfo.getClockName()
    def getClockEdge(self):
        if self.alwaysinfo is None: return ''
        return self.alwaysinfo.getClockEdge()
    def getClockBit(self):
        if self.alwaysinfo is None: return ''
        return self.alwaysinfo.getClockBit()
    def getResetName(self):
        if self.alwaysinfo is None: return ''
        return self.alwaysinfo.getResetName()
    def getResetEdge(self):
        if self.alwaysinfo is None: return ''
        return self.alwaysinfo.getResetEdge()
    def getResetBit(self):
        if self.alwaysinfo is None: return ''
        return self.alwaysinfo.getResetBit()
    def getSenslist(self):
        if self.alwaysinfo is None: return ''
        return self.alwaysinfo.getSenslist()

################################################################################
class DataFlow(object):
    def __init__(self):
        self.terms = {}
        self.binddict = {}

        self.functions = {}
        self.function_ports = {}
        self.tasks = {}
        self.task_ports = {}
        self.temporal_value = {}

    ############################################################################
    def addTerm(self, name, term):
        if not name in self.terms:
            self.terms[name] = term
        else:
            self.setTerm(name, term)

    def setTerm(self, name, term):
        self.terms[name].termtype |= term.termtype
        if self.terms[name].msb is None: self.terms[name].msb = term.msb
        if self.terms[name].lsb is None: self.terms[name].lsb = term.lsb
        if self.terms[name].lenmsb is None: self.terms[name].lenmsb = term.lenmsb
        if self.terms[name].lenlsb is None: self.terms[name].lenlsb = term.lenlsb

    def hasTerm(self, name):
        return name in self.terms

    def getTerm(self, name):
        if name in self.terms: return self.terms[name]
        return None

    def getTerms(self):
        return self.terms

    ############################################################################
    def addBind(self, name, bind):
        if name is None:
            raise verror.DefinitionError('Bind name is empty')
        if not name in self.binddict:
            self.binddict[name] = [bind,]
        else:
            self.setBind(name, bind)

    def setBind(self, name, bind):
        if name is None:
            raise verror.DefinitionError('Bind name is empty')
        currentbindlist = self.binddict[name]
        c_i = 0
        for c in currentbindlist:
            if c.msb == bind.msb and c.lsb == bind.lsb and c.ptr == bind.ptr:
                self.binddict[name][c_i].tree = bind.tree
                return
            c_i += 1
        self.binddict[name] = currentbindlist + [bind,]

    def getBindlist(self, name):
        if name in self.binddict: return tuple(self.binddict[name])
        return ()

    def getBinddict(self):
        return self.binddict

    ############################################################################
    def addFunction(self, name, definition):
        self.functions[name] = definition
    def hasFunction(self, name):
        return name in self.functions
    def getFunction(self, name):
        if name in self.functions: return self.functions[name]
        return None
    def addFunctionPorts(self, name, ports):
        self.function_ports[name] = ports
    def getFunctionPorts(self, name):
        if name in self.function_ports: return self.function_ports[name]
        return ()

    ############################################################################
    def addTask(self, name, definition):
        self.tasks[name] = definition
    def hasTask(self, name):
        return name in self.tasks
    def getTask(self, name):
        if name in self.tasks: return self.tasks[name]
        return None
    def addTaskPorts(self, name, ports):
        self.task_ports[name] = ports
    def getTaskPorts(self, name):
        if name in self.task_ports: return self.task_ports[name]
        return ()

    ############################################################################
    def setTemporalValue(self, name, value):
        self.temporal_value[name] = value
    def getTemporalValue(self, name):
        return self.temporal_value[name]




#-------------------------------------------------------------------------------
# splitter.py
#
# Splitting a definition tree into condition part and function part
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------
from __future__ import absolute_import
from __future__ import print_function
import sys
import os

import pyverilog.utils.signaltype as signaltype
from pyverilog.dataflow.dataflow import *

################################################################################
def split(tree):
    funcdict = {} # key:condition list, value:function
    if isinstance(tree, DFBranch):
        truefuncs = split(tree.truenode)
        truecond = tree.condnode
        if len(truefuncs) > 0:
            for cond, func in truefuncs.items():
                funcdict[(truecond,) + cond] = func
        else:
            if tree.truenode is not None:
                funcdict[(truecond,)] = tree.truenode
        falsefuncs = split(tree.falsenode)
        falsecond = DFOperator((tree.condnode,), 'Ulnot')
        if len(falsefuncs) > 0:
            for cond, func in falsefuncs.items():
                funcdict[(falsecond,) + cond] = func
        else:
            if tree.falsenode is not None:
                funcdict[(falsecond,)] = tree.falsenode
    return funcdict

################################################################################
def remove_reset_condition(funcdict):
    new_funcdict = {}
    for _condlist, func in funcdict.items():
        condlist = remove_reset_condlist(_condlist)
        new_funcdict[condlist] = func
    if () in new_funcdict and len(new_funcdict) > 1:
        del new_funcdict[ () ]
    return new_funcdict

def remove_reset_condlist(condlist):
    new_condlist = []
    for cond in condlist:
        r = _remove_reset_cond(cond)
        if r: new_condlist.append(r)
    return tuple(new_condlist)

def _remove_reset_cond(cond):
    condstr = cond.tostr()
    if signaltype.isReset(condstr): return None
    return cond

################################################################################
def active_constant(termname, node, op='>', value=0):
    if not isinstance(node, DFEvalValue): return False
    if not eval(str(node.value) + op + str(value)): return False
    return True

def active_modify(termname, node):
    if isinstance(node, DFTerminal) and node.name == termname: return False
    return True

def active_unmodify(termname, node):
    if isinstance(node, DFTerminal) and node.name == termname: return True
    return False

def filter(funcdict, termname, condition=active_constant):
    ret_funcdict = {}
    for condlist, func in funcdict.items():
        if not condition(termname, func): continue
        ret_funcdict[condlist] = func
    return ret_funcdict
